subroutine doboson(t, width, gauss, asym, emin, emax, wmin, wmax, np, p, debug, xout, yout, nout)

! *******************************************************************
! *                                                                 *
! * perform the quantum-mechanical complement to the classical step *
! * of the dielectric theory of eels in specular geometry using a   *
! * suitable thermodynamical average of the quantized surface       *
! * harmonic oscillators                                            *
! *                                                                 *
! * t:      Target temperature                                      *
! * width:  Width of instrumental response (cm**-1)                 *
! * gauss:  Fraction of gaussian for the instrumental response      *
! * asym:   Asymmetry of the instrumental response                  *
! * emin:   Lower loss energy for this computation (cm**-1)         *
! * emax:   Upper loss energy for this computation (cm**-1)         *
! * wmin:   Lower loss energy of the eels computation (cm**-1)      *
! * wmax:   Upper loss energy of the eels computation (cm**-1)      *
! * np:     Number of points of the eels computation                *
! * p:      Intensities of the eels computation                     *
! * xout:   Energies of this computation                            *
! * yout:   Intensities of this computation                         *
! * nout:   Number of points of this computation                    *
! *******************************************************************

  use rcffi_mod
  use sicot_mod
  use sintr_mod

  implicit none

  double precision, intent(in) :: t, width, gauss, asym, emin, emax, wmin
  double precision, intent(in out) :: wmax
  integer, intent(in) :: np
  double precision, intent(in) :: p(np)
  logical, intent(in) :: debug
  double precision, intent(in out) :: xout(nout), yout(nout)
  integer, intent(in out) :: nout

  integer, parameter :: mmax = 14, nmax = 2**mmax
  double precision :: a, a1, a2, alfa, anorm, b, cp2, dwn, fac, fi
  double precision :: fm, fm0, fm1, fp1, fmpic, fp, fp0, fppic, fr, g1, g2
  double precision :: h, pi, sigma, sp2, test, u
  double precision :: wmpic, wppic, wn, x, x1, x2, x3
  double precision :: o, respon

  external o

  integer :: i, imax, imin, istep, j, jmin, jmax, m, n
  logical :: picm, picp
  double precision, allocatable :: p1(:), p2(:)
  double precision :: r1(nmax), r2(nmax)

! remark : the two arrays r1 and r2 are used for fourier
! transforming the user-supplied instrumental response of the
! spectrometer that has to be coded in the external routine
! respon called when the input parameter gauss is < 0 or > 1.
! with 0 <= gauss <= 1, r1 and r2 are not used.

  data fm1 / 0.0d0 /, fp1 / 0.0d0 /
  pi = 4 * datan(1.0d0)

  dwn = (wmax - wmin) / (np - 1)

! *** redefine the frequency interval (0, wmax) to be used for the
! *** fourier transforms

  if (debug) write(*,*) 'wmin: ', wmin, 'wmax: ', wmax, 'dwn: ', dwn
  jmin = int(0.5d0 + wmin / dwn)
  if ((jmin - 0.5d0) * dwn < wmin) then
    jmin = jmin + 1
  endif
  fac = ((jmin - 0.5d0) * dwn - wmin) / dwn
  jmax = int(0.5d0 + wmax / dwn)
  test = dmax1(1.5d0 * dabs(emin), 1.5d0 * dabs(emax), 6 * wmax)
  n = 2
  do m = 2, mmax
    n = 2 * n
    wmax = n * dwn
    if (wmax >= test) exit
  enddo
  if (wmax < test) then
    m = mmax
    n = nmax
    wmax = n * dwn
    if (debug) write(*,*) ' +++ n has been fixed at', nmax, ' = 2**', mmax, ' +++'
  endif
  if (debug) then
    write(*,*) ' classical spectrum redefined from 0.0 to', wmax
    write(*,*) ' step size =', dwn, ', ', n, ' (= 2**', m, ') points'
  endif

  allocate (p1(n))
  allocate (p2(n))

  p1 = 0

! *** interpolate pcl on a suitable mesh in (0, wmax)
  i = 1
  do j = jmin, min(jmax, n)
    p1(j) = p(i) + fac * (p(i + 1) - p(i))
    i = i + 1
  enddo

  p2 = p1

! *** characteristic function f(tau)

  call sicot(p1, m, dwn, 1.39d0 * t)
  call sintr(p2, m, dwn)
  h = (pi + pi) / wmax
  if (gauss < 0.0d0 .or. gauss > 1.0d0) then

! *** broaden the spectrum by convoluting the characteristic function
! *** with a user-supplied response function (arbitrary normalization)

    if (debug) write(*,*) '==> switch to a user-supplied instrumental response'
    alfa = dmax1(4 * dwn, width)
    if (alfa > width) then
      if (debug) write(*,*) ' +++ width has been enlarged to', alfa, ' +++'
    endif
    if (alfa < 10 * dwn) then
      if (debug) then
        write(*,*) ' ... poor representation of the response function ...'
        write(*,*) 'the step size (', dwn, ') should be reduced'
      endif
    endif
! make a table of the response function
    r1(1) = respon(0.0d0, alfa)
    r2(1) = r1(1)
    do i = 2, n
      x = (i - 1) * dwn
      r1(i) = respon( x, alfa)
      r2(i) = respon(-x, alfa)
    enddo
! fourier transform it
    call rcffi(r1, r2, -m, dwn)
! normalization of the response function, and convolution
    anorm = r1(1)
    do i = 1, n
      fac = dexp(-p1(i)) / anorm
      cp2 = fac * dcos(p2(i))
      sp2 = fac * dsin(p2(i))
      p1(i) = r1(i) * cp2 + r2(i) * sp2
      p2(i) = r2(i) * cp2 - r1(i) * sp2
    enddo
    alfa = alfa / 2

  else

! *** broaden the spectrum by convoluting the characteristic function by
! *** a weighted sum of a lorentzian and a gaussian response functions

    alfa = dmax1(1.5d0 * dwn, width)
    if (alfa > width) then
      if (debug) write(*,*) ' +++ width has been enlarged to', alfa, ' +++'
    endif
    if (alfa > 0.5d0 * wmax / pi) then
      stop '*** width is too large, nothing done ***'
    endif
    alfa = 0.5d0 * alfa
    sigma = alfa / 1.66511d0
    a1 = (1.0d0 - asym) / 2 * (1.0d0 - gauss)
    a2 = (1.0d0 + asym) / 2 * (1.0d0 - gauss)
    if (a1 < 0.0d0 .or. a2 < 0.0d0) then
      stop '*** invalid input : asym should be in (-1, +1) ***'
    endif
    g1 = (1.0d0 - asym) * alfa
    g2 = (1.0d0 + asym) * alfa
    p1(1) = 1.0d0
    p2(1) = 0.0d0
    do i = 2, n
      x = (i - 1) * h
      fr = 0.0d0
      fi = 0.0d0
      if (a1 /= 0.0d0) then
        x1 = g1 * x
        if (x1 <= 100.0d0) then
          fr = a1 * dexp(-x1)
        endif
        if (a1 /= a2) then
          fi = a1 * o(x1)
        endif
      endif
      if (a2 /= 0.0d0) then
        x2 = g2 * x
        if (x2 <= 100.0d0) then
          fr = fr + a2 * dexp(-x2)
        endif
        if (a2 /= a1) then
          fi = fi - a2 * o(x2)
        endif
      endif
      if (gauss /= 0.0d0) then
        x3 = sigma * x
        if (x3 <= 10.0d0) then
          fr = fr + gauss * dexp(-x3**2)
        endif
      endif
      fi = fi / pi
      fac = dexp(-p1(i))
      cp2 = fac * dcos(p2(i))
      sp2 = fac * dsin(p2(i))
      p1(i) = fr * cp2 + fi * sp2
      p2(i) = fi * cp2 - fr * sp2
    enddo
    if (dabs(fi) > 1.0d-03) then
      if (debug) then
        write(*,*) ' ... poor representation of the response function ...'
        write(*,*) 'the step size (', dwn, ') should be reduced'
      endif
    endif
  endif

! *** full eels spectrum

   call rcffi(p1, p2, m, h)

! *** output

  istep = max(int(alfa / dwn / 10), 1)
  nout = 0
  if (emin <= 0.0d0) then
    imin = n - int(dabs(emin) / dwn)
    if ((imin - n) * dwn < emin) then
      imin = imin + 1
    endif
    do i = imin, n
      j = n - i
      if (mod(j, istep) > 0) cycle
      x = -j * dwn
      if (x > emax) exit
      nout = nout + 1
      xout(nout)= x
      yout(nout)= p2(j + 1)
    enddo
  endif
  if (x <= emax) then
    if (emax >= dwn) then
      imax = int(emax / dwn) + 1
      if ((imax - 1) * dwn > emax) then
        imax = imax - 1
      endif
      if (imax >= 2) then
        do i = 2, imax
          if (mod(i - 1, istep) > 0) cycle
          x = (i - 1) * dwn
          if (x < emin) cycle
          nout = nout + 1
          xout(nout) = x
          yout(nout) = p1(i)
        enddo
      endif
    endif
  endif
  if (debug) write(*,*) nout, ' values, step size =', istep * dwn

! *** analyze the spectrum

  if (debug) write(*,*) ' peak location    amplitude'
  wn = 0.0d0
  fm = p1(1)
  fp = p2(1)
  if (p2(2) < fp .and. p1(2) < fp) then
    if (debug) write(*, '(f13.2, e13.4)') wn, fp
  endif
  fac = 5.0d-06 * fp * dwn
  imax = 2 + int(dmax1(dabs(emin), dabs(emax)) / dwn)
  do i = 2, imax
    fm0 = fm1
    fp0 = fp1
    fm1 = fm
    fp1 = fp
    fm = p2(i)
    fp = p1(i)
    if (i == 2) cycle
    picm = .false.
    picp = .false.
    wn = (i - 1) * dwn
    if ((fm1 >= fm0) .and. (fm1 >= fm)) then
      a = (fm1 - fm0) + (fm1 - fm)
      if (a >= fac) then
        b = 0.5d0 * ((fm1 - fm0) + 3 * (fm1 - fm))
        u = b / a
        wmpic = -wn + u * dwn
        fmpic = fm + 0.5d0 * b * u
        picm = .true.
      endif
    endif
    if ((fp1 >= fp0) .and. (fp1 >= fp)) then
      a = (fp1 - fp0) + (fp1 - fp)
      if (a >= fac) then
        b = 0.5d0 * ((fp1 - fp0) + 3 * (fp1 - fp))
        u = b / a
        wppic = wn - u * dwn
        fppic = fp + 0.5d0 * b * u
        picp = .true.
        if (picp) then
          if (picm) then
            if (debug) write(*, '(2(f13.2, e13.4, 5x))') wppic, fppic, wmpic, fmpic
          else
            if (debug) write(*, '(f13.2, e13.4)') wppic, fppic
          endif
        endif
      endif
    endif
    if (picm .and. (.not. picp)) then
      if (debug) write(*, '(33x, f13.2, e13.4)') wmpic, fmpic
    endif
  enddo
  deallocate (p1)
  deallocate (p2)
  return
end subroutine doboson
