module quanc8_mod
contains
subroutine quanc8(fun, a, b, abserr, relerr, result, errest, nofun, flag, eps, thick, layers, nper)

! estimate the integral of fun(x) from a to b
! to a user provided tolerance.
! an automatic adaptive routine based on
! the 8-panel newton-cotes rule (g. forsythe et al, 1977, p. 92)
!
! input ..
!
! fun     the name of the integrand function subprogram fun(x).
! a       the lower limit of integration.
! b       the upper limit of integration.(b may be less than a.)
! relerr  a relative error tolerance. (should be non-negative)
! abserr  an absolute error tolerance. (should be non-negative)
!
! output ..
!
! result  an approximation to the integral hopefully satisfying the
!         least stringent of the two error tolerances.
! errest  an estimate of the magnitude of the actual error.
! nofun   the number of function values used in calculation of result.
! flag    a reliability indicator.  if flag is zero, then result
!         probably satisfies the error tolerance.  if flag is
!         xxx.yyy , then  xxx = the number of intervals which have
!         not converged and 0.yyy = the fraction of the interval
!         left to do when the limit on  nofun  was approached.
! 
! additional input ..
!
! eps     epsilon
! thick   thickness
! layers  number of layers
! nper    number of periodic layers

  implicit none

  double precision :: fun
  double precision, intent(in) :: a
  double precision, intent(in) :: b
  double precision, intent(in out) :: abserr
  double precision, intent(in) :: relerr
  double precision, intent(out) :: result
  double precision, intent(out) :: errest
  integer, intent(out) :: nofun
  double precision, intent(out) :: flag

  external fun

  double precision, intent(in) :: thick(:)
  double complex, intent(in) :: eps(:)
  integer, intent(in) :: layers, nper

  double precision :: w0, w1, w2, w3, w4, area, x0, f0, stone, step, cor11, temp
  double precision :: qprev, qnow, qdiff, qleft, esterr, tolerr
  double precision :: qright(31), f(16), x(16), fsave(8, 30), xsave(8, 30)
  double precision :: dabs, dmax1

  integer :: levmin, levmax, levout, nomax, nofin, lev, nim, i, j

! ***   stage 1 ***   general initialization
! set constants.

!  write (*,*) 'quanc8:'
!  write (*,*) 'thick: ', size(thick)
!  write (*,*) 'eps: ', size(eps)

  levmin = 1
  levmax = 30
  levout = 6
  nomax = 5000
  nofin = nomax - 8 * (levmax - levout + 2**(levout + 1))

! trouble when nofun reaches nofin

  w0 =   3956.0d0 / 14175.0d0
  w1 =  23552.0d0 / 14175.0d0
  w2 =  -3712.0d0 / 14175.0d0
  w3 =  41984.0d0 / 14175.0d0
  w4 = -18160.0d0 / 14175.0d0

! initialize running sums to zero.

  flag   = 0.0d0
  result = 0.0d0
  cor11  = 0.0d0
  errest = 0.0d0
  area   = 0.0d0
  nofun = 0
  if (a == b) return

! ***   stage 2 ***   initialization for first interval

  lev = 0
  nim = 1
  x0 = a
  x(16) = b
  qprev  = 0.0d0
  f0 = fun(x0, eps, thick, layers, nper, size(eps))
  stone = (b - a) / 16
  x(8)  =  (x0    + x(16)) / 2
  x(4)  =  (x0    + x(8))  / 2
  x(12) =  (x(8)  + x(16)) / 2
  x(2)  =  (x0    + x(4))  / 2
  x(6)  =  (x(4)  + x(8))  / 2
  x(10) =  (x(8)  + x(12)) / 2
  x(14) =  (x(12) + x(16)) / 2
!***  !$OMP PARALLEL shared (f, x, eps, thick, layers, nper) private (j)
!***  !$OMP DO
  do j = 2, 16, 2
    f(j) = fun(x(j), eps, thick, layers, nper, size(eps))
  enddo
!***  !$OMP END DO NOWAIT
!***  !$OMP END PARALLEL
  nofun = 9

  do

! ***   stage 3 ***   central calculation
! requires qprev, x0, x2, x4, ..., x16, f0, f2, f4, ..., f16.
! calculates x1, x3, ...x15, f1, f3, ...f15, qleft, qright, qnow, qdiff, area.

    x(1) = (x0 + x(2)) / 2
    f(1) = fun(x(1), eps, thick, layers, nper, size(eps))
    do j = 3, 15, 2
      x(j) = (x(j - 1) + x(j + 1)) / 2
    enddo
!***  !$OMP PARALLEL shared (f, x, eps, thick, layers, nper) private (j)
!***  !$OMP DO SCHEDULE(auto)
    do j = 3, 15, 2
      f(j) = fun(x(j), eps, thick, layers, nper, size(eps))
    enddo
!***  !$OMP END DO NOWAIT
!***  !$OMP END PARALLEL
    nofun = nofun + 8
    step = (x(16) - x0) / 16.0d0
    qleft  =  (w0 * (f0 + f(8))  + w1 * (f(1) + f(7)) + w2 * (f(2) + f(6))  &
        + w3 * (f(3) + f(5))  +  w4 * f(4)) * step
    qright(lev + 1) = (w0 * (f(8) + f(16)) + w1 * (f(9) + f(15)) + w2 * (f(10) + f(14))  &
        + w3 * (f(11) + f(13)) + w4 * f(12)) * step
    qnow = qleft + qright(lev + 1)
    qdiff = qnow - qprev
    area = area + qdiff

! ***   stage 4 *** interval convergence test

    esterr = dabs(qdiff) / 1023
    tolerr = dmax1(abserr, relerr * dabs(area)) * (step / stone)

    if (lev >= levmin) then
      if (lev >= levmax) then
! current level is levmax.
        flag = flag + 1.0d0
      else
        if (nofun > nofin) then
! ***   stage 6   ***   trouble section
! number of function values is about to exceed limit.
          nofin = 2 * nofin
          levmax = levout
          flag = flag + (b - x0) / (b - a)
        else
          if (esterr > tolerr) then
! ***   stage 5   ***   no convergence
! locate next interval.
            nim = 2 * nim
            lev = lev + 1
! store right hand elements for future use.
            do i = 1, 8
              fsave(i, lev) = f(i + 8)
              xsave(i, lev) = x(i + 8)
            enddo
! assemble left hand elements for immediate use.
            qprev = qleft
            do i = 1, 8
              f(18 - 2 * i) = f(9 - i)
              x(18 - 2 * i) = x(9 - i)
            enddo
            cycle
          endif
        endif
      endif

! ***   stage 7   ***   interval converged
! add contributions into running sums.
      result = result + qnow
      errest = errest + esterr
      cor11  = cor11  + qdiff / 1023
! locate next interval.
      do while (nim /= 2 * (nim / 2))
        nim = nim / 2
        lev = lev - 1
      enddo
      nim = nim + 1

      if (lev <= 0) exit

! assemble elements required for the next interval.
      qprev = qright(lev)
      x0 = x(16)
      f0 = f(16)
      do i = 1, 8
        f(2*i) = fsave(i, lev)
        x(2*i) = xsave(i, lev)
      enddo
      cycle
    else
! ***   stage 5   ***   no convergence
! locate next interval.
      nim = 2 * nim
      lev = lev + 1
! store right hand elements for future use.
      do i = 1, 8
        fsave(i, lev) = f(i + 8)
        xsave(i, lev) = x(i + 8)
      enddo
! assemble left hand elements for immediate use.
      qprev = qleft
      do i = 1, 8
        f(18 - 2 * i) = f(9 - i)
        x(18 - 2 * i) = x(9 - i)
      enddo
    endif

   enddo

 ! ***   stage 8   ***   finalize and return
  result = result + cor11

! make sure errest not less than roundoff level.
  if (errest /= 0.0d0) then
    temp = dabs(result) + errest
    do while (temp == dabs(result))
      errest = 2 * errest
      temp = dabs(result) + errest
    enddo
  endif
  return
end subroutine quanc8
end module quanc8_mod
